//screen left = 8, screen right = 247, paddle height = 32
//left paddle = 16/24, right paddle = 231/239
//top = 16, bottom = 240

//source file for Pong mini game
pong_main:
	gosub clear_palette
	gosub pong_initvars
	gosub pong_loadbg
pong_mainloop:
	if pong_loseball <> 0 gosub pong_lost_ball
	gosub pong_joyhandler
	gosub pong_physics
	gosub pong_speedhandler
	gosub pong_draw
	if pong_endgame = 0 branchto pong_mainloop
//show endscreen
	gosub vwait
	set $2006 $21
	set $2006 $c8
	set $2007 80 // p
	set $2007 76 // l
	set $2007 65 // a
	set $2007 89 // y
	set $2007 69 // e
	set $2007 82 // r
	set $2007 32
	set $2007 + pong_endgame 48
	set $2007 32
	set $2007 87 // w
	set $2007 73 // i
	set $2007 78 // n
	set $2007 83 // s
	set pong_endtimer 90
pong_endloop: //wait a bit before exiting
	gosub vwait
	dec pong_endtimer
	if pong_endtimer <> 0 branchto pong_endloop
	return

//speed it up every so often
pong_speedhandler:
	if pong_hits <> 4 return
	set pong_hits 0
	if pong_dx >= 128 dec pong_dx
	if pong_dx < 128 inc pong_dx
	return

//we lost a ball. take care of the score and losing
pong_lost_ball:
	gosub pong_losesound
	if pong_loseball = 1 then
		inc pong_score2
		if pong_score2 = 5 set pong_endgame 2
		endif
	if pong_loseball = 2 then
		inc pong_score1
		if pong_score1 = 5 set pong_endgame 1
		endif
	if pong_endgame = 0 then
		set pong_y 124
		set pong_x 124
		set pong_dy 0
		set pong_hits 0
		if pong_dx >= 128 set pong_dx 255
		if pong_dx < 128 set pong_dx 1
		endif
	set pong_loseball 0
	return

//init vars
pong_initvars:
	gosub clear_spritemem
	set pong_endgame 0
	set pong_loseball 0
	set pong_ballslost 0
	set pong_pad1y 112
	set pong_pad2y 112
	set pong_x 124
	set pong_y 128
	set pong_dx 255
	set pong_dy 0
	set pong_score1 0
	set pong_score2 0
	set pong_hits 0

	//y values
	set [spritemem 36] 32 //player 1 score
	set [spritemem 40] 32 //player 2 score

	//tile numbers
	set [spritemem 1] 4
	set [spritemem 5] 5
	set [spritemem 9] 5
	set [spritemem 13] 4
	set [spritemem 17] 4
	set [spritemem 21] 5
	set [spritemem 25] 5
	set [spritemem 29] 4
	set [spritemem 33] 6 //bouncing ball

	//attrib bytes
	set [spritemem 2] 1
	set [spritemem 6] 1
	set [spritemem 10] 1
	set [spritemem 14] %10000001 //flip vert
	set [spritemem 18] %01000001 //flip horiz
	set [spritemem 22] %01000001 //flip horiz
	set [spritemem 26] %01000001 //flip horiz
	set [spritemem 30] %11000001 //flip horiz + vert
	set [spritemem 34] %10 //ball
	set [spritemem 38] 0 //player 1 score
	set [spritemem 42] 0 //player 2 score

	//paddle x locations
	set [spritemem 3] 16
	set [spritemem 7] 16
	set [spritemem 11] 16
	set [spritemem 15] 16
	set [spritemem 19] 231
	set [spritemem 23] 231
	set [spritemem 27] 231
	set [spritemem 31] 231
	set [spritemem 39] 64 //player 1 score
	set [spritemem 43] 192 //player 2 score
	
	return

//handle joystick input
pong_joyhandler:
	//player 1
	gosub joystick1
	if joy1up = 1 if pong_pad1y > 8 then
		dec pong_pad1y
		dec pong_pad1y
		//dec pong_pad2y //for debug
		//dec pong_pad2y //for debug
		endif
	if joy1down = 1 if pong_pad1y < 212 then
		inc pong_pad1y
		inc pong_pad1y
		//inc pong_pad2y //for debug
		//inc pong_pad2y //for debug
		endif
	//player 2
	gosub joystick2
	if joy2up = 1 if pong_pad1y > 8 then
		dec pong_pad2y
		dec pong_pad2y
		endif
	if joy2down = 1 if pong_pad1y < 212 then
		inc pong_pad2y
		inc pong_pad2y
		endif
	return

//handle the bouncing ball
pong_physics:
	//vertical movement
	set pong_y + pong_y pong_dy
	if pong_y <= 16 if pong_dy >= 128 then
		set pong_y 16
		set pong_dy - 0 pong_dy
		endif
	if pong_y >= 224 if pong_dy < 128 then
		set pong_y 224
		set pong_dy - 0 pong_dy
		endif

	//horizontal movement
	set pong_x + pong_x pong_dx
	if pong_x < 24 then //collide with paddle 1?
		if pong_x > 16 if pong_y > - pong_pad1y 4 if pong_y < + pong_pad1y 38 then
			//hit paddle 1
			set x - pong_y - pong_pad1y 4
			set pong_dy [pong_results x]
			set pong_dx - 0 pong_dx
			gosub pong_playsound
			inc pong_hits
			return
			endif
		//Player 1 DIE!!
		set pong_loseball 1
		return
		endif
	if pong_x > 223 then //collide with paddle 2?
		if pong_x < 231 if pong_y > - pong_pad2y 4 if pong_y < + pong_pad2y 38 then
			//hit paddle 2
			set x - pong_y - pong_pad2y 4
			set pong_dy [pong_results x]
			set pong_dx - 0 pong_dx
			gosub pong_playsound
			inc pong_hits
			return
			endif
		//Player 2 DIE!!
		set pong_loseball 2
		return
		endif
	return

//set the sprite locs and draw scene
pong_draw:
	//draw the paddles
	set [spritemem 0] pong_pad1y
	set [spritemem 4] + pong_pad1y 8
	set [spritemem 8] + pong_pad1y 16
	set [spritemem 12] + pong_pad1y 24

	set [spritemem 16] pong_pad2y
	set [spritemem 20] + pong_pad2y 8
	set [spritemem 24] + pong_pad2y 16
	set [spritemem 28] + pong_pad2y 24

	//draw the ball
	set [spritemem 32] pong_y
	set [spritemem 35] pong_x

	//set the score
	set [spritemem 37] + pong_score1 48
	set [spritemem 41] + pong_score2 48

	gosub vwait
	set SPRITE_DMA 2
	return

//load the colors
pong_palette:
	set $2006 $3f
	set $2006 0
	set x 0
	pong_palette_1:
		set $2007 [pong_palette_data x]
		inc x
		if x <> 32 branchto pong_palette_1
	return	

//load a pretty background ?
pong_loadbg:
	//sprite 96
	gosub clear_background
	gosub vwait
	gosub pong_palette
	return

pong_playsound:
	//make a bouncing sound
	set $4000 %10001111
	set $4001 %00000000
	set $4002 %10101010
	set $4003 %11100001	
	//426 = 256 + 128 + 32 + 8 + 2 = %00110101010
	return

pong_losesound:
	//make a losing sound
	set $4000 %00001111
	set $4001 %10100011
	set $4002 %10101010
	set $4003 %11100001
	return

pong_results:
	asm
	;//20 numbers per side (40 total)
	.db 252, 252, 252, 252, 252, 253, 253, 253, 253, 253
	.db 254, 254, 254, 254, 254, 255, 255, 255, 255, 255
	.db 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 4, 4, 4, 4, 4
	endasm

pong_palette_data:
	asm
	.db $0e, $30, $00, $00, $0e, $12, $22, $00, $0e, $16, $15, $00, $0e, $00, $00, $00
	.db $0e, $30, $00, $00, $0e, $12, $22, $00, $0e, $16, $35, $00, $0e, $00, $00, $00
	endasm
